"use strict";

export default {
	production: {
		redis: {
			port: process.env.REDIS_PORT,
			host: process.env.REDIS_HOST,
			options: {
				auth_pass: process.env.REDIS_AUTHPASS // eslint-disable-line camelcase
			}
		},
		strategies: {
			local: {
				usernameField: "email",
				passwordField: "password"
			}
		},
		session: {
			proxy: process.env.SESSION_PROXY,
			secret: process.env.SESSION_SECRET,
			saveUninitialized: true,
			resave: false,
			rolling: true,
			name: "abl.id",
			cookie: {
				path: "/",
				httpOnly: true,
				secure: process.env.SESSION_COOKIE_SECURE,
				maxAge: 24 * 60 * 60 * 1000,
				signed: process.env.SESSION_COOKIE_SIGNED
			}
		}
	},
	development: {
		strategies: {
			local: {
				usernameField: "email",
				passwordField: "password"
			}
		},
		redis: {
			port: 6379,
			host: "localhost",
			options: {}
		},
		session: {
			proxy: false,
			secret: "keyboard_cat",
			saveUninitialized: true,
			resave: false,
			rolling: true,
			name: "abl.id",
			cookie: {
				path: "/",
				httpOnly: true,
				secure: false,
				maxAge: 24 * 60 * 60 * 1000,
				signed: false
			}
		}
	},
	test: {
		strategies: {
			local: {
				usernameField: "email",
				passwordField: "password"
			}
		},
		redis: {
			port: 6379,
			host: "localhost",
			options: {}
		},
		session: {
			proxy: false,
			secret: "keyboard_cat",
			saveUninitialized: true,
			resave: false,
			rolling: true,
			name: "abl.id",
			cookie: {
				path: "/",
				httpOnly: true,
				secure: false,
				maxAge: 24 * 60 * 60 * 1000,
				signed: false
			}
		}
	}
};
